﻿using System.Data.Common;
using Drive.Common.Data;
using Drive.Common.IntegrationTest.IntegrationDatabaseTest;
using FluentAssertions;
using NodaTime;
using Xunit;

namespace Drive.Common.IntegrationTest.Data
{
    [Collection(TestIntegrationDatabaseTestCollection.COLLECTION_NAME)]
    public class AggregateRootPersistorSuite
    {
        private TestIntegrationDatabaseTest _databaseTest;

        public AggregateRootPersistorSuite(TestIntegrationDatabaseTest databaseTest)
        {
            _databaseTest = databaseTest;
        }


        [Fact]
        public async Task modified_by_id_is_persisted()
        {
            using DbConnection connection = _databaseTest.OpenConnection();
            using DbTransaction transaction = connection.BeginTransaction();
            using DriveDbContext dbContext = _databaseTest.OpenDbContext(connection, transaction);
            AggregateRootPersister persistor = new AggregateRootPersister(dbContext, new DriveClockProvider());

            TestAggregateRootModel model = new()
            {
                CreatedAt = Instant.FromDateTimeUtc(DateTime.Now.ToUniversalTime()),
                CreatedById = 1,
                Id = Guid.NewGuid(),
                ModifiedAt = Instant.FromDateTimeUtc(DateTime.Now.ToUniversalTime()),
                ModifiedById = 2
            };

            DriveContext context = new DriveContext();
            context.DriveUserId = 3;


            await persistor.Persist(
                new TestAggregateRoot(model),
                context);

            transaction.Commit();

            model.ModifiedById.Should().Be(3);
        }
    }
}