﻿using System.Data.Common;
using Drive.Common.Data;
using Drive.Common.IntegrationTest.IntegrationDatabaseTest;
using FluentAssertions;
using Xunit;

namespace Drive.Common.IntegrationTest
{
    [CollectionDefinition(nameof(TestIntegrationDatabaseTestCollection))]
    public class TestIntegrationDatabaseTestCollection : ICollectionFixture<TestIntegrationDatabaseTest>
    {
        public const string COLLECTION_NAME = nameof(TestIntegrationDatabaseTestCollection);
    }

    [Collection(TestIntegrationDatabaseTestCollection.COLLECTION_NAME)]
    public class IntegrationDatabaseTestSuite
    {
        private TestIntegrationDatabaseTest _databaseTest;

        public IntegrationDatabaseTestSuite(TestIntegrationDatabaseTest databaseTest)
        {
            _databaseTest = databaseTest;
        }

        [Fact]
        public void can_read_existing_data()
        {
            TestModel test;

            {
                using DbConnection connection = _databaseTest.OpenConnection();
                using DbTransaction transaction = connection.BeginTransaction();
                using DriveDbContext dbContext = _databaseTest.OpenDbContext(connection, transaction);

                test = dbContext.Set<TestModel>()
                    .Where(x => x.Active && x.Code == _databaseTest.TestCode)
                    .Single();
            }

            test.Should().NotBeNull();
            test.Code.Should().Be(_databaseTest.TestCode);
            test.Active.Should().BeTrue();
        }

        [Fact]
        public void can_write_new_data()
        {
            var test = new TestModel()
            {
                Active = true,
                Code = "NEWTEST",
                Description = "New Test Object"
            };

            {
                using DbConnection connection = _databaseTest.OpenConnection();
                using DbTransaction transaction = connection.BeginTransaction();
                using DriveDbContext dbContext = _databaseTest.OpenDbContext(connection, transaction);

                dbContext.Add(test);
                dbContext.SaveChanges();

                transaction.Commit();
            }

            test.Id.Should().NotBe(0);
        }

        [Fact]
        public void can_read_existing_data_from_other_schema()
        {
            OtherSchemaModel test;

            {
                using DbConnection connection = _databaseTest.OpenConnection();
                using DbTransaction transaction = connection.BeginTransaction();
                using DriveDbContext dbContext = _databaseTest.OpenDbContext(connection, transaction);

                test = dbContext.Set<OtherSchemaModel>()
                    .Where(x => x.Active && x.Code == _databaseTest.TestCode)
                    .Single();
            }

            test.Should().NotBeNull();
            test.Code.Should().Be(_databaseTest.TestCode);
            test.Active.Should().BeTrue();
        }

        [Fact]
        public void can_write_new_data_to_other_schema()
        {
            var test = new OtherSchemaModel()
            {
                Active = true,
                Code = "OTHERTEST",
                Description = "Other New Test Object"
            };

            {
                using DbConnection connection = _databaseTest.OpenConnection();
                using DbTransaction transaction = connection.BeginTransaction();
                using DriveDbContext dbContext = _databaseTest.OpenDbContext(connection, transaction);

                dbContext.Add(test);
                dbContext.SaveChanges();

                transaction.Commit();
            }

            //assert
            test.Id.Should().NotBe(0);
        }
    }
}